//
//  TRMovieInspector.h
//  Tryst
//
//  Created by Francisco Tolmasky on Thu Nov 13 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import <AppKit/AppKit.h>
#import "TRMovie.h"


// Some text attribute definitions
#define kPropertyAttributeName          @"PropertyAttributeName"
#define kIdentifierAttributeName        @"IdentifierAttributeName"
#define kPlaceholderAttributeName 	@"PlaceholderAttributeName"
#define kLabelAttributeName             @"LabelAttributeName"
#define kNotFoundRange 			NSMakeRange(NSNotFound, 0)
#define kLeftTabLocation                85
#define kRightTabLocation               90

// Some shadow parameters
#define kShadowBorder 		15.0		/* Amount of space we leave to draw shadow, around the selection. */
#define kShadowBlurRadius	5.520833	/* NSShadow parameter for the shadow blur radius. */
#define kShadowOffsetY		-4.479167	/* NSShadow parameter for the shadow y offset. */
#define kShadowOffsetX		0.0		/* NSShadow parameter for the shadow x offset. */
#define kShadowBorderColor 	153.0f/255.0f   /* The color of the light border we draw next to the shadow. */

@class  TRMovieGutter,
        TRMovieImageView,
        TRMovieFileView;

@interface TRMovieInspector : NSTextView 
{
    // Model
    TRMutableMovie			*model;
    
    unsigned                currentEditedStartLocation;
    unsigned                currentEditedYearLocation;
    
    unsigned                temporaryIdentifierCount;

    // View
    TRMovieGutter           *gutter;
    
    TRMovieImageView        *coverArt;
    
    struct
    {
        NSImage             *image;
        NSPoint             origin;
        
        NSRange             lhsRange;
        NSRange             rhsRange;
    }                       rollover;
    
    IBOutlet NSPanel        *customLabelSheet;
    IBOutlet NSTextField    *customLabelTextField;
}

- (void)setModel:(TRMutableMovie *)aModel;
- (TRMutableMovie *)model;

- (void)setMovieGutter:(TRMovieGutter *)aMovieGutter;
- (TRMovieGutter *)movieGutter;

- (void)toggleEditable:(id)sender;

@end

@interface TRMovieInspector (Editing)

- (BOOL)isEditing;
- (void)endEditing;

- (NSRange)currentEditingGlyphRange;

@end

@interface TRMovieInspector (Selection)

- (NSRange)nextEditableRange:(unsigned)currentIndex forward:(BOOL)forward;

@end

@interface TRMovieInspector (Attributes)

- (NSDictionary *)labelAttributesForProperty:(NSString *)aProperty;
- (NSDictionary *)placeholderAttributesForProperty:(NSString *)aProperty
    identifier:(NSString *)anIdentifier
    placeholderString:(NSString *)aPlaceholderString
    font:(NSFont *)aFont;
- (NSDictionary *)typingAttributesForProperty:(NSString *)aProperty
    identifier:(NSString *)anIdentifier
    placeholderString:(NSString *)aPlaceholderString
    font:(NSFont *)aFont;
    
- (NSDictionary *)tabStopAttributesForScalarProperties;
- (NSDictionary *)tabStopAttributesForMultiValueProperties;

- (NSDictionary *)typingAttributes;

- (NSAttributedString *)fieldForProperty:(NSString *)aProperty
    identifier:(NSString *)anIdentifier
    selectedLabel:(NSString *)aLabel
    currentString:(NSString *)aCurrentString;
    
@end

@interface TRMovieInspector (Properties)

- (void)loadProperties;
- (void)recordValuesAndProperties;

- (IBAction)endCustomLabelSheet:(id)sender;

@end