//
//  TRBroadcasting.h
//  Tryst
//
//  Created by Francisco Tolmasky on Mon May 10 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import <Foundation/Foundation.h>

extern NSString * const             kTRBroadcastServerProperty;

extern NSString * const				TRBroadcastServerInformationDidChangeNotification;
extern NSString * const				TRBroadcastClientInformationDidChangeNotification;

typedef enum
{
	TRBroadcastSuccessfulResponse=		 0,
	TRBroadcastPasswordInvalidResponse= -1,
	TRBroadcastConnectionLimitResponse= -2
}   TRBroadcastResponse;

@class  TRMutableMovie,
		TRBroadcastClient;

@protocol TRBroadcastNodeProtocol
- (bycopy)uniqueId;
@end

@interface TRBroadcastNode : NSObject <TRBroadcastNodeProtocol>
{
	NSString		*uniqueId;
}
- (NSString *)uniqueId;
@end

@protocol TRBroadcastServerProtocol
- (bycopy)movie;
@end

@interface TRBroadcastServer : TRBroadcastNode <TRBroadcastServerProtocol>
{	
	// Model
	struct
	{
		TRMutableMovie  *movie;
	
		NSTimeInterval  timeRemaining;
		unsigned		connectionLimit;
		NSString		*password;
	}					broadcast;
	
	// Networking
	NSTimer				*timer;
	
	NSSocketPort		*socket;
	NSConnection		*connection;
	
	NSNetService		*service;
	NSMutableDictionary *clients;
	NSMutableDictionary *subscriptions;
	
	NSString			*multicast;
	BOOL				isPlaying;
}

- (BOOL)isPlaying;

- (TRMutableMovie *)movie;

- (NSTimeInterval)timeRemaining;

- (unsigned)connectionLimit;

- (unsigned)totalConnections;

- (void)broadcastMovie:(TRMutableMovie *)aMovie
	timeRemaining:(NSTimeInterval)aTimeInterval
	connectionLimit:(unsigned)aConnectionLimit
	password:(NSString *)aPassword;
	
- (BOOL)isPasswordProtected;

- (void)registerClient:(TRBroadcastClient *)aClient;
- (void)unregisterClient:(TRBroadcastClient *)aClient;
- (TRBroadcastResponse)subscribeClient:(TRBroadcastClient *)aClient withPassword:(NSString *)aPassword;

- (void)stop;

@end

@protocol TRBroadcastClientProtocol
- (oneway)receiveStreamWithMulticastAddress:(NSString *)multicastAddress name:(NSString *)movieName;
- (oneway)stopReceiving;
@end

@interface TRBroadcastClient : TRBroadcastNode <TRBroadcastClientProtocol>
{
	NSMutableArray		*movies;
	NSNetServiceBrowser *browser;
	
	TRBroadcastServer   *connectedServer;
}

- (NSArray *)movies;

- (void)close;
- (void)closeConnectionWithServer:(TRBroadcastServer *)aServer;
- (void)unsubscribe;

- (void)receiveStreamWithMulticastAddress:(NSString *)multicastAddress name:(NSString *)movieName;

- (void)setServer:(TRBroadcastServer *)aServer;
- (TRBroadcastServer *)server;

@end