//
//  TRController.m
//  Tryst
//
//  Created by Francisco Tolmasky on Thu Apr 08 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "NSFileManager+TRAdditions.h"
#import "TRMovie.h"

#import "TRBroadcasting.h"

#import "TRWindowController.h"
#import "TRController.h"


@implementation TRController

- (id)init
{
    self= [super init];
    if(self)
    {
		library= [[NSKeyedUnarchiver unarchiveObjectWithFile: [[[[NSFileManager safeSupportFolder] path] stringByAppendingString: @"/Tryst Movie Library"] stringByExpandingTildeInPath]] retain];
		if(!library) library= [[NSMutableArray alloc] init];
		
		server= [[TRBroadcastServer alloc] init];
		client= [[TRBroadcastClient alloc] init];
		
        windowController= [[TRWindowController alloc] initWithController: self];
		
		signal(SIGPIPE, SIG_IGN);
    }
    return self;
}

- (void)dealloc
{
	[server release];
	[client release];
}

- (TRWindowController *)windowController
{
	return windowController;
}

- (NSMutableArray *)library
{
	return library;
}

- (TRBroadcastServer *)server
{
	return server;
}

- (TRBroadcastClient *)client
{
	return client;
}

- (void)removeLostMovie:(TRMovie *)aMovie
{
	[[client movies] removeObject: aMovie];
	
	[[NSNotificationCenter defaultCenter]
		postNotification: TRBroadcastClientInformationDidChangeNotification
		object: self];
	
}

@end

@implementation TRController (NSApplicationDelegate)

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    [windowController window];//load window.
    [windowController showWindow: self];
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
	[client close];
	[server stop];

	[NSKeyedArchiver archiveRootObject: library toFile: [[[[NSFileManager safeSupportFolder] path] stringByAppendingString: @"/Tryst Movie Library"] stringByExpandingTildeInPath]];

    [windowController applicationWillTerminate: aNotification];
}

- (void)workspaceWillPowerOff:(NSNotification *)aNotification
{
	[client close];
	[server stop];

	[NSKeyedArchiver archiveRootObject: library toFile: [[[[NSFileManager safeSupportFolder] path] stringByAppendingString: @"/Tryst Movie Library"] stringByExpandingTildeInPath]];

	[windowController applicationWillTerminate: aNotification];
}

@end