//
//  TRIntegerControl.m
//  Tryst
//
//  Created by Francisco Tolmasky on Mon Apr 12 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRIntegerControl.h"


@implementation TRIntegerControl


- (id)initWithFrame:(NSRect)aFrame
{
	self= [super initWithFrame: aFrame];
	if(self)
	{
		NSNumberFormatter *integerFormatter= [[NSNumberFormatter alloc] init];
		[integerFormatter setFormat: @"00"];
		
		integerCell= [[NSTextFieldCell alloc] initTextCell: @"03"];
		[integerCell setDrawsBackground: NO];
		[integerCell setEditable: YES];
		[integerCell setFont: [NSFont labelFontOfSize: 13.0]];
		[integerCell setFormatter: integerFormatter];
		
		[integerFormatter release];
		
		stepper= [[NSStepper alloc] initWithFrame: NSMakeRect([integerCell cellSize].width+10.5,0.0,19.0,27.0)];
		[stepper setTarget: integerCell];
		[stepper setAction: @selector(takeIntValueFrom:)];
		
		[stepper setMinValue: 1];
		[stepper setMaxValue: 99];
		[stepper setIncrement: 1];
		[stepper setIntValue: 3];
		
		[self addSubview: stepper];
		
		[self setFrameSize: NSMakeSize(NSMaxX([stepper frame]),26.0)];
		
		[self setRefusesFirstResponder: NO];
	}
	return self;
}

- (void)setIntValue:(int)aValue
{
	[stepper setIntValue: aValue];
	[integerCell setIntValue: aValue];
}

- (int)intValue
{
	return [stepper intValue];
}

- (void)setEnabled:(BOOL)flag
{
	[integerCell setTextColor: flag?[NSColor blackColor]:[NSColor disabledControlTextColor]];
	
	[integerCell setEnabled: flag];
	[stepper setEnabled: flag];
	
	[super setEnabled: flag];
}

- (BOOL)acceptsFirstResponder
{
	return [self isEnabled];
}

- (void)mouseDown:(NSEvent *)theEvent
{
	[super mouseDown: theEvent];
	
	if([self isEnabled]) [[self window] makeFirstResponder: self];
}

- (void)drawRect:(NSRect)clipRect
{
	[[self isEnabled]?[NSColor grayColor]:[NSColor lightGrayColor] set];
	
	NSRect cellBounds= NSMakeRect(3.5,6.0,[integerCell cellSize].width,[integerCell cellSize].height);
	
	[NSBezierPath strokeLineFromPoint: NSMakePoint(NSMinX([self bounds])+0.5,ceilf(NSMaxY(cellBounds))+1.0) 
                              toPoint: NSMakePoint(NSMinX([self bounds])+0.5,5.0)];

	[NSBezierPath strokeLineFromPoint: NSMakePoint(ceilf(NSMinX([self bounds]))+0.5,ceilf(NSMaxY(cellBounds))+1.0-0.5) 
                              toPoint: NSMakePoint(ceilf(NSMaxX(cellBounds)+3.5)-1.5,ceilf(NSMaxY(cellBounds))+1.0-0.5)];

	[NSBezierPath strokeLineFromPoint: NSMakePoint(ceilf(NSMaxX(cellBounds)+3.5)-1.5,ceilf(NSMaxY(cellBounds))+1.0) 
                              toPoint: NSMakePoint(ceilf(NSMaxX(cellBounds)+3.5)-1.5,5.0)];
	
	[NSBezierPath strokeLineFromPoint: NSMakePoint(ceilf(NSMinX([self bounds]))+0.5,5.5) 
                              toPoint: NSMakePoint(ceilf(NSMaxX(cellBounds)+3.5)-1.5,5.5)];
    
	[[NSColor whiteColor] set];
	NSRectFill(NSMakeRect(1.0,6.0,ceilf(NSMaxX(cellBounds)+3.5)-3.0,cellBounds.size.height));
	
	if([[self window] firstResponder]==self)
	{
		if([[self window] isKeyWindow]) [[NSColor selectedTextBackgroundColor] set];
		else [[NSColor secondarySelectedControlColor] set];
		NSRectFill(cellBounds);
	}

	[integerCell drawWithFrame: cellBounds inView: self];
}

@end
