//
//  TRMovieCell.m
//  Tryst
//
//  Created by Francisco Tolmasky on Fri Nov 28 2003.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRMovie.h"
#import "TRBroadcasting.h"

#import "TRMovieCell.h"


NSDictionary                *TRMovieTitleAttributes;
NSDictionary                *TRMovieTitleHighlightedAttributes;
NSDictionary                *TRMovieGenreAttributes;
NSDictionary                *TRMovieGenreHighlightedAttributes;
NSDictionary				*TRMovieTimeIntervalAttributes;
NSDictionary				*TRMovieTimeIntervalHighlightedAttributes;

NSMutableAttributedString   *TRMovieWorkingString;

@implementation TRMovieCell

+ (void)initialize
{
    if(self==[TRMovieCell class])
    {
        NSMutableParagraphStyle *truncatingParagraphStyle=
            [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
        [truncatingParagraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];

        TRMovieTitleAttributes= [[NSDictionary
            dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 12.0], truncatingParagraphStyle, nil]
            forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSParagraphStyleAttributeName, nil]] retain];
        TRMovieTitleHighlightedAttributes= [[NSDictionary
            dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 12.0], [NSColor whiteColor], truncatingParagraphStyle, nil]
            forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, NSParagraphStyleAttributeName, nil]] retain];
        TRMovieGenreAttributes= [[NSDictionary
            dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 10.0], [NSColor grayColor], truncatingParagraphStyle, nil]
            forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, NSParagraphStyleAttributeName, nil]] retain];
        TRMovieGenreHighlightedAttributes= [[NSDictionary
            dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 10.0], [NSColor whiteColor], truncatingParagraphStyle, nil]
            forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, NSParagraphStyleAttributeName, nil]] retain];
			
		TRMovieTimeIntervalAttributes= [[NSDictionary
            dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 11.0], [NSColor blackColor], nil]
            forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, nil]] retain];
		TRMovieTimeIntervalHighlightedAttributes= [[NSDictionary
            dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 11.0], [NSColor whiteColor], nil]
            forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, nil]] retain];
    }
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    //[[self controlView] setNeedsDisplay: YES];
}

/*- (void)setObjectValue:(id)anObject
{
    TRMovie *previous= [self objectValue];
    
    if(previous)
    {
        NSLog([previous valueForProperty: kTRMovieTitleProperty]);
        [previous removeObserver: self forKeyPath: kTRMovieTitleProperty];
        [previous removeObserver: self forKeyPath: kTRMovieYearProperty];
        [previous removeObserver: self forKeyPath: kTRMovieGenreProperty];
    }
    
    [super setObjectValue: anObject];
    
    [anObject
        addObserver: self forKeyPath: kTRMovieTitleProperty
            options: (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context: NULL];
    [anObject
        addObserver: self forKeyPath: kTRMovieYearProperty
            options: (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context: NULL];
    [anObject
        addObserver: self forKeyPath: kTRMovieGenreProperty
            options: (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context: NULL];
}*/

- (void)drawInteriorWithFrame:(NSRect)cellFrame
    inView:(NSView *)controlView
	movie:(TRMovie *)aMovie;
{
	NSString	*rawTitle= [aMovie valueForProperty: kTRMovieTitleProperty],
				*year= [aMovie valueForProperty: kTRMovieYearProperty],
                *genre= [aMovie valueForProperty: kTRMovieGenreProperty];

	NS_DURING
	
	TRBroadcastServer  *server= [aMovie valueForProperty: kTRBroadcastServerProperty];
	
	NSAttributedString *title;
	
    BOOL useHighlight=  [self isHighlighted]&&
                        [[[self controlView] window] isKeyWindow]&&
                        [[[self controlView] window] firstResponder]==[self controlView];
	
	if(server)
	{
		NSTimeInterval timeRemaining= [server timeRemaining];
		unsigned	totalConnections= [server totalConnections],
					connectionLimit= [server connectionLimit];
		
		BOOL isPlaying;
		if(isPlaying= timeRemaining<=0) timeRemaining= -timeRemaining;

		NSAttributedString *timeIntervalString= [[NSAttributedString alloc] initWithString:
			[NSString stringWithFormat: @"%02d:%02d", (int)timeRemaining/3600, ((int)timeRemaining%3600)/60]
			attributes: useHighlight?TRMovieTimeIntervalHighlightedAttributes:TRMovieTimeIntervalAttributes];
		
		NSAttributedString *connectionsString= [[NSAttributedString alloc] initWithString: 
			connectionLimit?[NSString stringWithFormat: @"%d/%d", totalConnections, connectionLimit]:[NSString stringWithFormat: @"%d",totalConnections]
			attributes: TRMovieTimeIntervalHighlightedAttributes];
		
		NSRect broadcastStatusRect= NSMakeRect(cellFrame.origin.x+3.0,cellFrame.origin.y+5.0,[connectionsString size].width+20.0,[connectionsString size].height+1);
		
		float   capRadius= broadcastStatusRect.size.height/2,
				timex= cellFrame.origin.x+broadcastStatusRect.size.width;
		
		NSBezierPath *path= [NSBezierPath bezierPath];
		
		[path setLineWidth: 2.0];
		
		[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMaxX(broadcastStatusRect)+2.0,NSMinY(broadcastStatusRect)+capRadius) radius: capRadius startAngle: 270.0f endAngle: 360.0f];// clockwise: YES];
		[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMaxX(broadcastStatusRect)+2.0,NSMaxY(broadcastStatusRect)-capRadius) radius: capRadius startAngle: 0.0f endAngle: 90.0f];// clockwise: YES];
		[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMinX(broadcastStatusRect)+capRadius,NSMaxY(broadcastStatusRect)-capRadius) radius: capRadius startAngle: 90.0f endAngle: 180.0f];
		[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMinX(broadcastStatusRect)+capRadius,NSMinY(broadcastStatusRect)+capRadius) radius: capRadius startAngle: 180.0f endAngle: 270.0f];
	
		[path closePath];
		
		NSAffineTransform *transform= [NSAffineTransform transform];
		[transform translateXBy: -0.3 yBy: 0.5];
		[path transformUsingAffineTransform: transform];
		
		[[NSColor grayColor] set];
		
		[path setLineWidth: 1.0];
		
		[path fill];
		
		NSAffineTransform *transform2= [NSAffineTransform transform];
		[transform2 translateXBy: 0.3 yBy: -0.7];
		[path transformUsingAffineTransform: transform2];
		if([[[NSApp delegate] client] server]==server)
			[[NSColor colorWithCalibratedRed: 66/255.0f green: 184/255.0f blue: 7/255.0f alpha: 1.0f] set];
		else if(isPlaying)
			[[NSColor colorWithCalibratedRed: 222/255.0f green: 18/255.0f blue: 18/255.0f alpha: 1.0f] set];
		else
			[[NSColor colorWithCalibratedRed: 51/255.0f green: 115/255.0f blue: 241/255.0f alpha: 1.0f] set];
	
		[path fill];
		
		[[NSImage imageNamed: @"Broadcast Status"] compositeToPoint: NSMakePoint(cellFrame.origin.x+7.0,cellFrame.origin.y+18.0) operation: NSCompositeSourceOver];
		
		[connectionsString drawAtPoint: NSMakePoint(27.0,cellFrame.origin.y+5.0)];
	
		[useHighlight?[NSColor whiteColor]:[NSColor blackColor] set];
		
		[NSBezierPath setDefaultLineWidth: 1.3];
		
		[[NSBezierPath bezierPathWithOvalInRect: NSMakeRect(timex+16.0,cellFrame.origin.y+6.0,12.0,12.0)] stroke];
		
		[NSBezierPath setDefaultLineWidth: 0.6];
		[NSBezierPath strokeLineFromPoint: NSMakePoint(timex+17.0,cellFrame.origin.y+14.0)
			toPoint: NSMakePoint(timex+3.0+19.0,cellFrame.origin.y+12.0)];
		[NSBezierPath strokeLineFromPoint: NSMakePoint(ceilf(timex+3.0+19.0)-0.5,cellFrame.origin.y+8.0)
			toPoint: NSMakePoint(ceilf(timex+3.0+19.0)-0.5,cellFrame.origin.y+12.0)];
		[timeIntervalString drawAtPoint: NSMakePoint(timex+16.0+16.0,cellFrame.origin.y+5.0)];
	}
	
	NSRect titleRect= NSMakeRect(cellFrame.origin.x+2.0, cellFrame.origin.y+(server?22.0:2.0), 
						cellFrame.size.width, cellFrame.size.height);
	
	if(!rawTitle)
	{
		title= [[NSMutableAttributedString alloc]
			initWithString: @"Movie Title"];
			//attributes: useHighlight?TRMovieTitleHighlightedAttributes:TRMovieTitleAttributes];
		[(NSMutableAttributedString *)title applyFontTraits: NSItalicFontMask range: NSMakeRange(0,11)];
	}
	else title= [[NSMutableAttributedString alloc] initWithString: rawTitle
		attributes: useHighlight?TRMovieTitleHighlightedAttributes:TRMovieTitleAttributes];
	
	if(year)
	{
		NSString *p_year= [NSString stringWithFormat: @" (%@)",year];
	
		NSSize  t_size= [title size],
				y_size= [p_year sizeWithAttributes: TRMovieTitleAttributes];
		
		titleRect.size.width-= y_size.width;
		
		float width= (t_size.width<titleRect.size.width)?t_size.width:titleRect.size.width;
		if(t_size.width>titleRect.size.width) p_year= [NSString stringWithFormat: @"(%@)",year];
    
		[p_year drawAtPoint: NSMakePoint(titleRect.origin.x+width,titleRect.origin.y)
			withAttributes: useHighlight?TRMovieTitleHighlightedAttributes:TRMovieTitleAttributes];
	}
	
	[title drawInRect: titleRect];
	
    if(genre)
		[genre
			drawInRect: NSMakeRect(cellFrame.origin.x+2.0,cellFrame.origin.y+(server?38.0:18.0),cellFrame.size.width-2.0,cellFrame.size.height)
			withAttributes: useHighlight?TRMovieGenreHighlightedAttributes:TRMovieGenreAttributes];
	
    NS_HANDLER
    
	[[NSApp delegate] removeLostMovie: aMovie];
	
	NS_ENDHANDLER
	
}

- (void)drawInteriorWithFrame:(NSRect)aFrame
	inView:(NSView *)controlView
{
	[self drawInteriorWithFrame: aFrame inView: controlView movie: [self objectValue]];
}

@end
