//
//  TRMovieInspector(Properties).m
//  Tryst
//
//  Created by Francisco Tolmasky on Fri Dec 26 2003.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRMultiValue.h"
#import "TRMovie.h"

#import "TRMovieLabelCell.h"
#import "TRMovieGutter.h"
#import "TRMovieScrollView.h"
#import "TRMovieFileView.h"
#import "TRMovieInspector.h"


@implementation TRMovieInspector (Properties)

- (NSString *)currentEditedProperty
{
    if(currentEditedStartLocation!=NSNotFound)
        return [[self textStorage] attribute: kPropertyAttributeName atIndex: currentEditedStartLocation effectiveRange: NULL];
    return nil;
}

- (NSString *)currentEditedIdentifier
{
    if(currentEditedStartLocation!=NSNotFound)
        return [[self textStorage] attribute: kIdentifierAttributeName atIndex: currentEditedStartLocation effectiveRange: NULL];
    return nil;
}

- (NSString *)nextTemporaryIdentifier
{
    return [NSString stringWithFormat: @"t_%i", temporaryIdentifierCount++];
}

- (BOOL)loadValuesForProperty:(NSString *)aProperty;
{
    TRMultiValue *propertyList= [model valueForProperty: aProperty];

    NSTextStorage *textStorage= [self textStorage];
    
    unsigned    i= 0,
                length= [propertyList count];
    
    NSString *identifier;
    
    if([self isEditable] && !length)
    {
        [textStorage appendAttributedString: [self fieldForProperty: aProperty identifier: identifier= [self nextTemporaryIdentifier] selectedLabel: nil currentString: nil]];
        //[[(TRMovieScrollView *)[self enclosingScrollView] gutter] appendValueControlForProperty: aProperty identifier: identifier isFinal: YES];//doesn't matter (YES YES)
        return YES;
    }
    for(;i<length;++i)
    {
        [textStorage appendAttributedString:
            [self fieldForProperty: aProperty identifier: identifier= [propertyList identifierAtIndex: i] selectedLabel: [propertyList labelAtIndex: i] currentString: [propertyList valueAtIndex: i]]];
        //[[(TRMovieScrollView *)[self enclosingScrollView] gutter] appendValueControlForProperty: aProperty identifier: identifier isFinal: i==length-1];
    }
    
    return length;
}

- (void)loadProperties
{
    if(!model) return;

    NSTextStorage *textStorage= [self textStorage];
    NSAttributedString *tab= [[[NSAttributedString alloc] initWithString: @"\t"] autorelease];
    NSAttributedString *newline= [[[NSAttributedString alloc] initWithString: @"\n"] autorelease];

    unsigned x;

    // Clean up code (pre) first.
    [self setSelectedRange: NSMakeRange(0,0)];
    temporaryIdentifierCount= 0;
    
	NSString	*title= [model valueForProperty: kTRMovieTitleProperty],
				*year= [model valueForProperty: kTRMovieYearProperty],
				*genre= [model valueForProperty: kTRMovieGenreProperty];
	
    [textStorage replaceCharactersInRange: NSMakeRange(0,[textStorage length]) withString: @""];
    
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: tab];
    [textStorage appendAttributedString: tab];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: title?title:@"Movie Title"
        attributes: title?
			[self typingAttributesForProperty: kTRMovieTitleProperty identifier: @"" placeholderString: @"Movie Title" font: [NSFont boldSystemFontOfSize: 14]]:
			[self placeholderAttributesForProperty: kTRMovieTitleProperty identifier: @"" placeholderString: @"Movie Title" font: [NSFont boldSystemFontOfSize: 14]]]];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: @"  (" 
        attributes: [NSDictionary dictionaryWithObject: [NSFont boldSystemFontOfSize: 14] forKey: NSFontAttributeName]]];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: year?year:@"Year"
        attributes: year?
			[self typingAttributesForProperty: kTRMovieYearProperty identifier: @"" placeholderString: @"Year" font: [NSFont boldSystemFontOfSize: 14]]:
			[self placeholderAttributesForProperty: kTRMovieYearProperty identifier: @"" placeholderString: @"Year" font: [NSFont boldSystemFontOfSize: 14]]]];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: @") "
        attributes: [NSDictionary dictionaryWithObject: [NSFont boldSystemFontOfSize: 14] forKey: NSFontAttributeName]]];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: tab];
    [textStorage appendAttributedString: tab];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: genre?genre:@"Genre"
        attributes: genre?
			[self typingAttributesForProperty: kTRMovieGenreProperty identifier: @"" placeholderString: @"Genre" font: [NSFont systemFontOfSize: 11]]:
			[self placeholderAttributesForProperty: kTRMovieGenreProperty identifier: @"" placeholderString: @"Genre" font: [NSFont systemFontOfSize: 11]]]];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: newline];
    
    [textStorage removeAttribute: NSParagraphStyleAttributeName range: NSMakeRange(0,[textStorage length])];
    [textStorage addAttributes: [self tabStopAttributesForScalarProperties] range: NSMakeRange(0,x=[textStorage length])];    
    
    if([self loadValuesForProperty: kTRMovieCrewProperty])
        [[self textStorage] appendAttributedString: [[[NSAttributedString alloc] initWithString: @"\n"] autorelease]];
    [self loadValuesForProperty: kTRMovieCastProperty];
    
    [textStorage removeAttribute: NSParagraphStyleAttributeName range: NSMakeRange(x,[textStorage length]-x)];
    [textStorage addAttributes: [self tabStopAttributesForMultiValueProperties] range: NSMakeRange(x,[textStorage length]-x)];
}                                                                                                         

- (void)recordValuesAndProperties
{
    if(!model || ![[self textStorage] length]) return;
    
    NSTextStorage *textStorage= [self textStorage];
    NSRange effectiveRange= NSMakeRange(3,1);
    
    NSTextAttachment *attachment;
    BOOL fieldIsValid;
    
    TRMutableMultiValue *multiValueBuffer= nil;
    
    [textStorage attribute: kPropertyAttributeName atIndex: effectiveRange.location effectiveRange: &effectiveRange];
    [textStorage attribute: kPropertyAttributeName atIndex: NSMaxRange(effectiveRange)+3 effectiveRange: &effectiveRange];
    [textStorage attribute: kPropertyAttributeName atIndex: NSMaxRange(effectiveRange)+5 effectiveRange: &effectiveRange];
    
    effectiveRange= NSMakeRange(NSMaxRange(effectiveRange)+5,1);
    
    while(effectiveRange.location<[textStorage length])
    {
        attachment= [textStorage attribute: NSAttachmentAttributeName atIndex: effectiveRange.location effectiveRange: &effectiveRange];
        fieldIsValid= [textStorage attribute: NSForegroundColorAttributeName atIndex: NSMaxRange(effectiveRange)+1 effectiveRange: &effectiveRange]==[NSColor blackColor];
        if(fieldIsValid)
        {
            if(!multiValueBuffer) multiValueBuffer= [[TRMutableMultiValue alloc] init];
            [multiValueBuffer
                addValue: [[textStorage string] substringWithRange: effectiveRange]
                withLabel: [[(TRMovieLabelCell *)[attachment attachmentCell] selectedItem] title]];
        }
        
        effectiveRange.location= NSMaxRange(effectiveRange)+2;
    
        if(effectiveRange.location>=[textStorage length] || [[textStorage string] characterAtIndex: effectiveRange.location]=='\t')
        {
            if(multiValueBuffer)
            {
                [model setValue: multiValueBuffer forProperty: [(TRMovieLabelCell *)[attachment attachmentCell] representedProperty]];
                [multiValueBuffer release];
                multiValueBuffer= nil;
            }
            else [model removeValueForProperty: [(TRMovieLabelCell *)[attachment attachmentCell] representedProperty]];
            effectiveRange.location+= 1;
        }
    }
}

- (void)didChangeTextAtLocation:(unsigned)editedLocation
{
    NSTextStorage *textStorage= [self textStorage];
    NSTextContainer *textContainer= [self textContainer];
    
    NSRange effectiveRange;
    
    float height;
    
    [super didChangeText];
    
    if(editedLocation==NSNotFound) return;
    
    NSString *property= [textStorage attribute: kPropertyAttributeName atIndex: editedLocation effectiveRange: &effectiveRange];
    NSString *identifier= [textStorage attribute: kIdentifierAttributeName atIndex: editedLocation effectiveRange: NULL];
    
    if(property==kTRMovieTitleProperty || property==kTRMovieYearProperty || property==kTRMovieGenreProperty)
    {
        if([textStorage attribute: NSForegroundColorAttributeName atIndex: editedLocation effectiveRange: NULL]==[NSColor grayColor])
			[model removeValueForProperty: property];
		else
			[model setValue: [[textStorage string] substringWithRange: effectiveRange] forProperty: property];
        
        effectiveRange= NSMakeRange(3,1);
        
        [textStorage attribute: kPropertyAttributeName atIndex: effectiveRange.location effectiveRange: &effectiveRange];
        [textStorage attribute: kPropertyAttributeName atIndex: NSMaxRange(effectiveRange)+3 effectiveRange: &effectiveRange];
        [textStorage attribute: kPropertyAttributeName atIndex: NSMaxRange(effectiveRange)+5 effectiveRange: &effectiveRange];
        
        [textStorage attribute: kPropertyAttributeName atIndex: NSMaxRange(effectiveRange)+7 effectiveRange: &effectiveRange];
        
        height= [[textContainer layoutManager] boundingRectForGlyphRange:  [[self layoutManager] glyphRangeForCharacterRange: effectiveRange actualCharacterRange: NULL] inTextContainer: textContainer].origin.y;
    }
    else height= [[textContainer layoutManager] boundingRectForGlyphRange:  [[self layoutManager] glyphRangeForCharacterRange: effectiveRange actualCharacterRange: NULL] inTextContainer: textContainer].size.height;
    
    BOOL isPlaceholder= [textStorage attribute: NSForegroundColorAttributeName atIndex: effectiveRange.location+1 effectiveRange: NULL]==[NSColor grayColor];
    
    if(editedLocation== currentEditedStartLocation)
        [[(TRMovieScrollView *)[self enclosingScrollView] gutter]
            updateActiveValueControlWithValue: isPlaceholder?nil:[[textStorage string] substringWithRange: effectiveRange] height: height];
    else
        [[(TRMovieScrollView *)[self enclosingScrollView] gutter] updateValueControlForProperty: property identifier: identifier 
            withValue: isPlaceholder?nil:[[textStorage string] substringWithRange: effectiveRange] height: height];
}

- (void)didChangeText
{
    [self didChangeTextAtLocation: currentEditedStartLocation];
}

// Warning: DO NOT CALL didChangeText:
// Reason: Associated control not created yet, undefined results.
- (NSString *)appendFieldFollowingFieldAtPoint:(NSPoint)aPoint
{
    NSLayoutManager *layoutManager= [self layoutManager];
    NSTextStorage *textStorage= [self textStorage];
    
    unsigned glyphIndex= [layoutManager glyphIndexForPoint: aPoint inTextContainer: [self textContainer]];
    unsigned characterIndex= [layoutManager characterIndexForGlyphAtIndex: glyphIndex];
        
    NSRange effectiveRange;
        
    NSString    *property= [[self textStorage] attribute: kPropertyAttributeName atIndex: characterIndex+2 effectiveRange: &effectiveRange],
                *identifier;
    NSAttributedString *field;
	
    effectiveRange= NSMakeRange(NSMaxRange(effectiveRange)+1,0);

	[self setSelectedRange: NSMakeRange(0,0)]; //ARRRRGHHGHGHG!!!! Error without this, move down, then no where to select.
	
	[textStorage replaceCharactersInRange: effectiveRange withAttributedString:
        field= [self fieldForProperty: property identifier: identifier= [self nextTemporaryIdentifier] selectedLabel: nil currentString: nil]];
    [textStorage addAttributes: [self tabStopAttributesForMultiValueProperties] range: NSMakeRange(effectiveRange.location,[field length])];
    
    [self setSelectedRange: NSMakeRange(effectiveRange.location+3,0)];

    return identifier;
}

- (void)clearFieldAtPoint:(NSPoint)aPoint
{
    NSLayoutManager *layoutManager= [self layoutManager];
    NSTextStorage *textStorage= [self textStorage];
    
    unsigned glyphIndex= [layoutManager glyphIndexForPoint: aPoint inTextContainer: [self textContainer]];
    unsigned characterIndex= [layoutManager characterIndexForGlyphAtIndex: glyphIndex];
    
    NSRange effectiveRange;
    
    NSString    *property= [textStorage attribute: kPropertyAttributeName atIndex: characterIndex+2 effectiveRange: &effectiveRange],
                *identifier= [textStorage attribute: kIdentifierAttributeName atIndex: effectiveRange.location effectiveRange: NULL];
    
    NSAttributedString *a_str= [[NSAttributedString alloc] initWithString: @"Names" attributes:
        [self placeholderAttributesForProperty: property
            identifier: identifier
            placeholderString: @"Names"
            font: [NSFont systemFontOfSize: 11.0]]];
    
    [self setSelectedRange: NSMakeRange(0,0)];
    [textStorage replaceCharactersInRange: effectiveRange withAttributedString: a_str];
    [self setSelectedRange: NSMakeRange(effectiveRange.location,0)];
    
    [self didChangeText];
}

- (NSString *)removeFieldAtPoint:(NSPoint)aPoint
{
    NSLayoutManager *layoutManager= [self layoutManager];
    NSTextStorage *textStorage= [self textStorage];
    
    unsigned glyphIndex= [layoutManager glyphIndexForPoint: aPoint inTextContainer: [self textContainer]];
    unsigned characterIndex= [layoutManager characterIndexForGlyphAtIndex: glyphIndex];
    
    NSRange effectiveRange;
    
    [textStorage attribute: kPropertyAttributeName atIndex: characterIndex+2 effectiveRange: &effectiveRange];
    
    [self setSelectedRange: NSMakeRange(effectiveRange.location-5,0)];
    [textStorage replaceCharactersInRange: NSMakeRange(effectiveRange.location-4,effectiveRange.length+4) withString: @""];
    
    [self didChangeText];
	
	return [textStorage attribute: NSForegroundColorAttributeName atIndex: [self selectedRange].location effectiveRange: &effectiveRange]==[NSColor grayColor]?
		nil:[[textStorage string] substringWithRange: effectiveRange];
}

- (void)updateLayoutForAttachment:(NSTextAttachment *)anAttachment
{
    //- (void)showAttachmentCell:(NSCell *)cell inRect:(NSRect)rect characterIndex:(unsigned)attachmentIndex
    [[self layoutManager] textContainerChangedGeometry: [self textContainer]];//invalidateDisplayForCharacterRange: NSMakeRange(0,[[self string] length])];
    [self setNeedsDisplay: YES];
}

- (void)runCustomLabelSheet:(id)sender
{
    [customLabelTextField setStringValue: @"new label"];
    [customLabelTextField selectText: self];
    
    [NSApp beginSheet: customLabelSheet
        modalForWindow: [self window]
        modalDelegate: self
        didEndSelector: @selector(customLabelSheetDidEnd:returnCode:movieLabelCell:)
        contextInfo: sender];
}

- (IBAction)endCustomLabelSheet:(id)sender
{
    [NSApp endSheet: customLabelSheet returnCode: [sender tag]];
}

- (void)customLabelSheetDidEnd:(NSWindow *)aSheet returnCode:(int)returnCode movieLabelCell:(TRMovieLabelCell *)movieLabelCell
{
    if(returnCode && [[customLabelTextField stringValue] length])
    {
        NSTextStorage *textStorage= [self textStorage];
		NSString *string= [self string];
		
		[movieLabelCell addCustomLabel: [customLabelTextField stringValue] select: YES];
        //[self updateLayoutForAttachment: [movieLabelCell attachment]];
		
		int i= 0,
			length= [textStorage length];
		
		for(;i<length;++i)
			if([string characterAtIndex: i]==NSAttachmentCharacter && [textStorage attribute: NSAttachmentAttributeName atIndex: i effectiveRange: NULL]==[movieLabelCell attachment])
			{
				if([self selectedRange].location!=i+2) [self setSelectedRange: NSMakeRange(i+2,0)];
				//[[self layoutManager] invalidateDisplayForCharacterRange: NSMakeRange(i,1)];
				break;
			}
	}
	
    [customLabelSheet orderOut: self];
}

@end
