//
//  TRMovieScrollView.m
//  Tryst
//
//  Created by Francisco Tolmasky on Fri Dec 26 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "TRMultiValue.h"
#import "TRMovie.h"

#import "TRMovieGutter.h"
#import "TRMovieFileView.h"
#import "TRMovieInspector.h"
#import "TRMovieScrollView.h"

@interface TRMovieInspector (friend)
- (NSString *)nextTemporaryIdentifier;
- (NSString *)appendFieldFollowingIdentifier:(NSString *)anIdentifier;
- (void)clearFieldForIdentifier:(NSString *)anIdentifier;
- (BOOL)removeFieldForIdentifier:(NSString *)anIdentifier;
@end

@implementation TRAuxiliaryClipView

// ======================================================================
// setDocumentView:
// - Removes self as an observer for the NSViewBoundsChangedNotification 
//   and NSViewFrameDidChangeNotification to avoid unecessary and 
//   extrenious notifications and messages.
// ======================================================================

- (void)setDocumentView:(NSView *)aView
{
    [super setDocumentView: aView];
    [[NSNotificationCenter defaultCenter]
        removeObserver: self
        name: NSViewBoundsDidChangeNotification
        object: [self documentView]];
    [[NSNotificationCenter defaultCenter]
        removeObserver: self
        name: NSViewFrameDidChangeNotification
        object: [self documentView]];
}

@end

@implementation TRMovieScrollView

- (id)initWithCoder:(NSCoder *)aCoder
{
    self= [super initWithCoder: aCoder];
    if(self)
    {
        isEvaluatingNecessityOfScroller= NO;
        
        gutter= [[TRMovieGutter alloc] initWithFrame: NSMakeRect(0.0,0.0,kGutterWidth,[self contentSize].height-87.0)];
        
        [(TRMovieInspector *)[self documentView] setMovieGutter: gutter];
        
        gutterClipView= [[TRAuxiliaryClipView alloc] initWithFrame: NSMakeRect(-kGutterWidth,0.0,kGutterWidth,[self frame].size.height)];
        [gutterClipView setDocumentView: gutter];
        
		fileView= [[TRMovieFileView alloc] initWithFrame: NSMakeRect(0.0,0.0,100.0,40.0)];
		
		[fileView setMovieInspector: [self documentView]];
		
		[fileView
			addObserver: self forKeyPath: @"hidden"
				options: (NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld) context: NULL];
		
        [self addSubview: gutterClipView];
		[self addSubview: fileView];
		
		[gutter release];
		[gutterClipView release];
		
		[fileView release];
    }
    return self;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([keyPath isEqual: @"hidden"]) [self tile];
}

- (TRMovieFileView *)fileView
{
	return fileView;
}

- (TRMovieGutter *)gutter
{
    return gutter;
}

- (void)showGutter:(id)sender
{
    [gutterClipView setFrameOrigin: NSMakePoint(1.0,1.0)];
    [self tile];
    [self setNeedsDisplay: YES];
}

- (void)hideGutter:(id)sender
{
    [gutterClipView setFrameOrigin: NSMakePoint(-kGutterWidth,1.0)];
    [self tile];
    [self setNeedsDisplay: YES];
}

- (void)updateGutterHeight
{
    [gutter setFrameSize: NSMakeSize(kGutterWidth,[[self documentView] frame].size.height)];
}

- (void)evaluateNecessityOfVerticalScroller
{
    if(isEvaluatingNecessityOfScroller) return;
    isEvaluatingNecessityOfScroller= YES;
    BOOL isNecessary= [[self contentView] documentVisibleRect].size.height<[[self documentView] frame].size.height;
    if(isNecessary!=[self hasVerticalScroller]) [self setHasVerticalScroller: isNecessary];
    isEvaluatingNecessityOfScroller= NO;
}

- (void)setFrameSize:(NSSize)aSize
{
    [super setFrameSize: aSize];
	[self tile]; // Avoid jump
    [self evaluateNecessityOfVerticalScroller];
}

- (void)tile
{	
    //[super tile];
    
	NSRect contentRect= NSMakeRect(1.0,1.0,0.0,0.0);
	contentRect.size= [NSScrollView 
		contentSizeForFrameSize: [self frame].size 
		hasHorizontalScroller: NO 
		hasVerticalScroller: [self hasVerticalScroller] 
		borderType: [self borderType]];
	
	if(![fileView isHidden])//[(TRMovieInspector *)[self documentView] model]) // for now
	{
		NSRect fileRect;
		
		NSDivideRect(contentRect,&fileRect,&contentRect,40.0,NSMaxYEdge);
		if([self hasVerticalScroller]) fileRect.size.width+= [NSScroller scrollerWidth];
		[fileView setFrame: fileRect];
	}
	
	if([self hasVerticalScroller])
		[[self verticalScroller] setFrame: NSMakeRect([self frame].size.width-[NSScroller scrollerWidth]-1.0,1.0,[NSScroller scrollerWidth],contentRect.size.height)];
	
    if([gutterClipView frame].origin.x!=-kGutterWidth)
    {
		NSRect gutterRect;
		
        NSDivideRect(contentRect,&gutterRect,&contentRect,kGutterWidth,NSMinXEdge);
        [gutterClipView setFrame: gutterRect];
    }
	
	[[self contentView] setFrame: contentRect];
	
    //scrollerRect= [[self horizontalScroller] frame];
    //[gutterButton setFrame:
    //    NSMakeRect(scrollerRect.origin.x,scrollerRect.origin.y,WSGUTTER_WIDTH,15.0)];
    //scrollerRect.size.width-= (scrollerRect.origin.x+= WSGUTTER_WIDTH);
    //[[self horizontalScroller] setFrame: scrollerRect];*/
}

- (void)scrollClipView:(NSClipView *)aClipView toPoint:(NSPoint)newOrigin
{
    if(aClipView!=gutterClipView)
    {
        [super scrollClipView: aClipView toPoint: newOrigin];
        [super scrollClipView: gutterClipView toPoint: NSMakePoint(0.0,[aClipView documentVisibleRect].origin.y)];
    }
}

@end