//
//  TRTimeIntervalControl.m
//  Tryst
//
//  Created by Francisco Tolmasky on Sun Apr 11 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRTimeIntervalControl.h"


@implementation TRTimeIntervalControl

- (id)initWithFrame:(NSRect)aFrame
{
	self= [super initWithFrame: aFrame];
	if(self)
	{
		timeInterval= 60;
		
		hoursCell= [[NSTextFieldCell alloc] initTextCell: @"0 hours"];
		[hoursCell setDrawsBackground: NO];
		[hoursCell setEditable: YES];
		[hoursCell setFont: [NSFont labelFontOfSize: 13.0]];
		
		minutesCell= [[NSTextFieldCell alloc] initTextCell: @"10 minutes"];
		[minutesCell setDrawsBackground: NO];
		[minutesCell setEditable: YES];
		[minutesCell setFont: [NSFont labelFontOfSize: 13.0]];
		
		bounds.hours= NSMakeRect(3.5,8.0,[hoursCell cellSize].width,[hoursCell cellSize].height);
		bounds.minutes= NSMakeRect(bounds.hours.size.width+7.5,8.0,[minutesCell cellSize].width,[minutesCell cellSize].height);
		
		editing.cell= minutesCell;
		editing.bounds= bounds.minutes;
		
		stepper= [[NSStepper alloc] initWithFrame: NSMakeRect(NSMaxX(bounds.minutes)+7.0,2.0,19.0,27.0)];
		[stepper setTarget: self];
		[stepper setAction: @selector(takeTimeIntervalFrom:)];
		
		[stepper setMaxValue: ((int)timeInterval/3600+1)*3600-60];
		[stepper setMinValue: ((int)timeInterval/3600)*3600+60];
		[stepper setIncrement: 60];
		[stepper setDoubleValue: timeInterval];
		
		[self addSubview: stepper];
		
		[self setFrameSize: NSMakeSize(NSMaxX([stepper frame]),28.0)];
		
		[self setRefusesFirstResponder: NO];
	}
	return self;
}

- (id)target
{
	return target;
}

- (void)setTarget:(id)aTarget
{
	target= aTarget;
}

- (void)setAction:(SEL)anAction
{
	action= anAction;
}

- (SEL)action
{
	return action;
}

- (void)setTimeInterval:(NSTimeInterval)aTimeInterval
{	
	timeInterval= aTimeInterval;
	
	int hours= (int)timeInterval/3600,
		minutes= ((int)timeInterval%3600)/60;
	
	[stepper setDoubleValue: timeInterval];
	[hoursCell setStringValue: hours==1?@"1 hour":[NSString stringWithFormat: @"%d hours", hours]];
	[minutesCell setStringValue: minutes==1?@"01 minute":[NSString stringWithFormat: @"%02d minutes", minutes]];
	
	[self sendAction: [self action] to: [self target]];
}

- (NSTimeInterval)timeInterval
{
	return timeInterval;
}

- (IBAction)takeTimeIntervalFrom:(id)sender
{	
	if([sender doubleValue]<60)
	{
		[sender setDoubleValue: 60];
		[stepper setMinValue: 60];
		[stepper setMaxValue: 9*3600+60];
	}
	
	[self setTimeInterval: [sender doubleValue]];
}

- (void)mouseDown:(NSEvent *)theEvent
{
	[super mouseDown: theEvent];
	
	[[self window] makeFirstResponder: self];
	
    NSPoint localPoint= [self convertPoint: [theEvent locationInWindow] fromView: nil];
    
	if(NSPointInRect(localPoint,bounds.minutes))
	{
		editing.cell= minutesCell;
		editing.bounds= bounds.minutes;
		
		int hours= (int)timeInterval/3600;
		
		[stepper setMaxValue: (hours+1)*3600-60];
		[stepper setMinValue: hours?(hours*3600):60];
		[stepper setIncrement: 60];
		[stepper setDoubleValue: timeInterval];
		
		
	}
	else
	{
		editing.cell= hoursCell;
		editing.bounds= bounds.hours;
		
		int minutes= (int)timeInterval%3600;
		
		[stepper setMinValue: minutes?minutes:3600];
		[stepper setMaxValue: 9*3600+minutes];
		[stepper setIncrement: 3600];
		[stepper setDoubleValue: timeInterval];
	}
	
	[self setNeedsDisplay: YES];
}

- (void)drawRect:(NSRect)clipRect
{
	[[NSColor grayColor] set];
	
	[NSBezierPath strokeLineFromPoint: NSMakePoint(NSMinX([self bounds])+0.5,ceilf(NSMaxY(bounds.minutes))+1.0) 
                              toPoint: NSMakePoint(NSMinX([self bounds])+0.5,7.0)];

	[NSBezierPath strokeLineFromPoint: NSMakePoint(ceilf(NSMinX([self bounds]))+0.5,ceilf(NSMaxY(bounds.minutes))+1.0-0.5) 
                              toPoint: NSMakePoint(ceilf(NSMaxX(bounds.minutes)+3.5)-1.5,ceilf(NSMaxY(bounds.minutes))+1.0-0.5)];

	[NSBezierPath strokeLineFromPoint: NSMakePoint(ceilf(NSMaxX(bounds.minutes)+3.5)-1.5,ceilf(NSMaxY(bounds.minutes))+1.0) 
                              toPoint: NSMakePoint(ceilf(NSMaxX(bounds.minutes)+3.5)-1.5,7.0)];
	
	[NSBezierPath strokeLineFromPoint: NSMakePoint(ceilf(NSMinX([self bounds]))+0.5,7.5) 
                              toPoint: NSMakePoint(ceilf(NSMaxX(bounds.minutes)+3.5)-1.5,7.5)];
    
	[[NSColor whiteColor] set];
	NSRectFill(NSMakeRect(1.0,8.0,ceilf(NSMaxX(bounds.minutes)+3.5)-3.0,bounds.hours.size.height));
	
	if([[self window] firstResponder]==self)
	{
		if([[self window] isKeyWindow]) [[NSColor selectedTextBackgroundColor] set];
		else [[NSColor secondarySelectedControlColor] set];
		NSRectFill(NSMakeRect(editing.bounds.origin.x,editing.bounds.origin.y,[editing.cell cellSize].width,[editing.cell cellSize].height));
	}

	[hoursCell drawWithFrame: NSMakeRect(3.5,8.0,bounds.hours.size.width,bounds.hours.size.height) inView: self];
	[minutesCell drawWithFrame: NSMakeRect(bounds.hours.size.width+7.5,8.0,bounds.minutes.size.width,bounds.minutes.size.height) inView: self];
	
	[[NSColor grayColor] set];
	
	[NSBezierPath setDefaultLineWidth: 2.0];
    
    [NSBezierPath strokeLineFromPoint: NSMakePoint(floor(NSMidX(editing.bounds))-4,0.0) 
                              toPoint: NSMakePoint(floor(NSMidX(editing.bounds)),4.0)];
    [NSBezierPath strokeLineFromPoint:NSMakePoint(floor(NSMidX(editing.bounds)),4.0)
                              toPoint:NSMakePoint(floor(NSMidX(editing.bounds))+4,0.0)];
							  
	[NSBezierPath setDefaultLineWidth: 1.0];
}

@end
